--[[
 ############################################################################
 #
 #  FILE: functions.lua
 #
 #  DESCRIPTION:  holds shared lua utility functions
 #
 # Copyright © 1994–2013 Lua.org, PUC-Rio.
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
 # to deal in the Software without restriction, including without limitation
 # the rights to use, copy, modify, merge, publish, distribute, sublicense,
 # and/or sell copies of the Software, and to permit persons to whom the
 # Software is furnished to do so, subject to the following conditions:
 #
 # The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
 #
 # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 # OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 # MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 # IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 # CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 # TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 # SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 #
 #  Copyright ©2011-2013 Motorola Mobilit, Inc.  All rights reserved
 #  Copyright ©2013-2015 Arris Enterprises, Inc.  All rights reserved
 #
 #  This program is confidential and proprietary to ARRIS Enterprises, Inc.
 #  (ARRIS), and may not be copied, reproduced, modified, disclosed to others,
 #  published or used, in whole or in part, without the expressed prior written
 #  permission of ARRIS.
 #
 ############################################################################
--]]
module(..., package.seeall);

-- Used to escape "'s by toCSV
function escapeCSV (s)
  if string.find(s, '[,"]') then
    s = '"' .. string.gsub(s, '"', '""') .. '"'
  end
  return s
end

-- Used to find the mode ATM or PTM
function isAtm()
    local sku = sdb.get("system.sku")
    local mode = sdb.get(string.format("system.sku-%s.cur-mode", sku))
    local msg
    if mode == "adsl"  then
        msg = "yes"
    else
        msg = "no"
    end
    return msg
end

-- Convert from CSV string to table
function fromCSV (s)
  s = s .. ','        -- ending comma
  local t = {}        -- table to collect fields
  local fieldstart = 1
  repeat
    -- next field is quoted? (start with `"'?)
    if string.find(s, '^"', fieldstart) then
      local a, c
      local i  = fieldstart
      repeat
        -- find closing quote
        a, i, c = string.find(s, '"("?)', i+1)
      until c ~= '"'    -- quote not followed by quote?
      if not i then error('unmatched "') end
      local f = string.sub(s, fieldstart+1, i-1)
      table.insert(t, (string.gsub(f, '""', '"')))
      fieldstart = string.find(s, ',', i) + 1
    else                -- unquoted; find next comma
      local nexti = string.find(s, ',', fieldstart)
      table.insert(t, string.sub(s, fieldstart, nexti-1))
      fieldstart = nexti + 1
    end
  until fieldstart > string.len(s)
  return t
end

-- Convert from table to CSV string
function toCSV (tt)
  local s = ""
  for _,p in pairs(tt) do
    s = s .. "," .. escapeCSV(p)
  end
  return string.sub(s, 2)      -- remove first comma
end

-- generate ordered lists using any type of object
-- if it is a string which represents a number then do the right thing
function cmp_multitype(op1, op2)
    local n1, n2, A1, A2

    if op1 == nil or op2 == nil then return true end
    if op1 == "" or op2 == "" then return true end
    local type1 = type(op1)
    local type2 = type(op2)
    if type1 ~= type2 then --cmp by type
        return type1 < type2
    elseif type1 == "number" and type2 == "number" then
        return op1 < op2
    elseif type1 == "string" and type2 == "string" then
        -- see if whole string is a number
        n1 = tonumber(op1)
        n2 = tonumber(op2)
        if n1 ~= nil and n2 ~= nil then
           return n1 < n2
        end

        -- ip_address [<space> netmask ]
        --    first substitute x for all numbers and dots then looks for anything except and 'x' in C
        a1, b1 = string.gsub(op1, "%d+%.*", "x")
            c1 = string.find(a1,"[^x]")
        a2, b2 = string.gsub(op2, "%d+%.*", "x")
            c2 = string.find(a1,"[^x]")
        if (c1 == nil or c1 == 5) and (c2 == nil or c2 == 5) then
            -- substitute dots for commas then compare on each digit
            a1,  _ = string.gsub(op1, "%.", ",")
            a2,  _ = string.gsub(op2, "%.", ",")
            a1,  _ = string.gsub(a1, " ", ",")
            a2,  _ = string.gsub(a2, " ", ",")
             _, c1 = string.gsub(a1, "%d+%,*", "x")
             _, c2 = string.gsub(a2, "%d+%,*", "x")

            -- now create tables from Comma Seperated Value strings a1 and a2
            A1 = fromCSV(a1)
            A2 = fromCSV(a2)

            for i=1,4 do
                n1 = tonumber(A1[i])
                n2 = tonumber(A2[i])
                -- did number conversion fail?
                if n1 == nil then
                   n1 = 0
                end
                if n2 == nil then
                   n2 = 0
                end
                -- convert out or bounds to in bounds
                if n1 > 255 or n1 < 0 then n1 = 0 end
                if n2 > 255 or n2 < 0 then n2 = 0 end
                -- compare
                if n1 ~= n2 then
                    -- get out if one of the digits is different
                    -- SESS.SESS_error_message = string.format("N1: %s <br/> N2: %s <br/>",n1,n2)
                    return n1 < n2
                end
            end

            -- compare the netmask, note that these are done in reverse order, larger subnet masks go first
            if b1==8 and b2==8 and c1==8 and c2==8 then
                for i=5,8 do
                   n1 = tonumber(A1[i])
                   n2 = tonumber(A2[i])
                   -- did number conversion fail?
                   if n1 == nil then
                     n1 = 0
                   end
                   if n2 == nil then
                     n2 = 0
                   end
                   -- convert out or bounds to in bounds
                   if n1 > 255 or n1 < 0 then n1 = 255 end
                   if n2 > 255 or n2 < 0 then n2 = 255 end

                   -- compare, note that here bigger goes 1st and lower goes last, just as route decision would be made.
                   if n1 ~= n2 then
                      return n1 > n2
                   end
                end
                return 0
            end
        end
        -- see if ip6_address[<space>netmask] look for colons
        -- 2DO

        -- see if ascii + ending number
        c1,d1=string.find(op1, "%d+$")
        c2,d2=string.find(op2, "%d+$")
        if c1 ~= nil and c2 ~= nil and c1 < # op1 and c2 < # op2 then
            -- compare ascii first
           a1 = string.sub(op1, 1, c1-1)
           a2 = string.sub(op2, 1, c2-1)
	   if a1 ~= a2 then
               return a1 < a2
           end
           -- compare numbers
           n1 = tonumber(string.sub(op1,c1, d1))
           n2 = tonumber(string.sub(op2,c2, d2))
           -- did number conversion fail?
           if n1 == nil then
             n1 = 0
           end
           if n2 == nil then
             n2 = 0
           end
           return n1 < n2
        end

        -- default just compare
        return op1 < op2

    elseif type1 == "boolean" and type2 == "boolean" then
        return op1 == true
    else
        return tostring(op1) < tostring(op2) --cmp by address
    end
end


-- create an ordering of the key values in orderedIndex list
function __genOrderedIndex( t )
    local orderedIndex = {}
    for key in pairs(t) do
        table.insert( orderedIndex, key )
    end
    table.sort( orderedIndex, cmp_multitype )
    return orderedIndex
end
function orderedNext(t, state)
    -- Equivalent of the next function, but returns the keys in the alphabetic
    -- order. We use a temporary ordered key table that is stored in the
    -- table being iterated.

    if state == nil then
        -- the first time, generate the index
        t.__orderedIndex = __genOrderedIndex( t )
        key = t.__orderedIndex[1]
        return key, t[key]
    end
    -- fetch the next value
    key = nil
    for i = 1,table.getn(t.__orderedIndex) do
        if t.__orderedIndex[i] == state then
            key = t.__orderedIndex[i+1]
        end
    end

    if key then
        return key, t[key]
    end

    -- no more value to return, cleanup
    t.__orderedIndex = nil
    return
end
function orderedPairs(t)
    -- Equivalent of the pairs() function on tables. Allows to iterate
    -- in order
    return orderedNext, t, nil
end

function sess_save()
  if FORM.nextpage == nil then
    SESS["SESS_END_PAGE"] = ENV.REQUEST_URI
  end
  if (ENV.AUTH_OK ~= nil) then
    if ENV.REQUEST_URI ~= "/cgi-bin/language.ha" and
      ENV.REQUEST_URI ~= "/cgi-bin/lang_de.ha" and
      ENV.REQUEST_URI ~= "/cgi-bin/lang_en.ha" and
      ENV.REQUEST_URI ~= "/cgi-bin/lang_fr.ha" and
      ENV.REQUEST_URI ~= "/cgi-bin/lang_it.ha" and
      ENV.REQUEST_URI ~= "/cgi-bin/login.ha" and
      ENV.REQUEST_URI ~= "/cgi-bin/logout.ha" and
      ENV.REQUEST_URI ~= "/cgi-bin/cancel.ha" and
      ENV.REQUEST_URI ~= "/cgi-bin/ipdiags.ha" and
      ENV.REQUEST_URI ~= "/cgi-bin/redirect.ha" and
      ENV.SCRIPT_NAME ~= "/cgi-bin/menuchange.ha" and
      ENV.REQUEST_URI ~= "/cgi-bin/menuconfig.ha" and
      ENV.REQUEST_URI ~= "/cgi-bin/menustatistics.ha" and
      ENV.REQUEST_URI ~= "/cgi-bin/menuutilities.ha" and
      ENV.REQUEST_URI ~= "/cgi-bin/menumedia.ha" and
      ENV.REQUEST_URI ~= "/cgi-bin/menuWAN.ha" and
      ENV.REQUEST_URI ~= "/cgi-bin/menuLAN.ha" and
      ENV.REQUEST_URI ~= "/cgi-bin/menuVoice.ha" and
      ENV.REQUEST_URI ~= "/cgi-bin/menuRouting.ha" and
      ENV.REQUEST_URI ~= "/cgi-bin/menuSWAN.ha" and
      ENV.REQUEST_URI ~= "/cgi-bin/menuSLAN.ha" and
      ENV.REQUEST_URI ~= "/cgi-bin/menuSRouting.ha" and
      ENV.REQUEST_URI ~= "/cgi-bin/menuAppHosting.ha" then
        SESS["SESS_SAFE_PAGE"] = ENV.REQUEST_URI
    end
  end
end

function sess_restore()
  if SESS == nil then
    functions.redirect(string.format("/cgi-bin/busy.ha?%s", ENV.REQUEST_URI))
  end
  if SESS["SESS_END_PAGE"] ~= nil then
      -- don't change it if we haven't changed page
      -- don't change it if we're on the menu page
      if ((SESS["SESS_END_PAGE"]  ~= ENV.REQUEST_URI)  and
          (SESS["SESS_END_PAGE"]  ~=  "/cgi-bin/language.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/cancel.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/lang_de.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/lang_en.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/lang_fr.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/lang_it.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/gethpna.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/hpnawarn.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/login.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/logout.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/ipdiags.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/redirect.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/wrestart.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/vrestart.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/crestart.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/hrestart.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/menuconfig.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/menustatistics.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/menuutilities.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/menumedia.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/menuWAN.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/menuLAN.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/menuVoice.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/menuRouting.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/menuSWAN.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/menuSLAN.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/menuSRouting.ha") and
          (SESS["SESS_END_PAGE"]  ~= "/cgi-bin/menuAppHosting.ha")) then
           SESS["SESS_LAST_PAGE"] = SESS["SESS_END_PAGE"]
      elseif ENV.REQUEST_URI ~= "/cgi-bin/commit.ha" and sdb.get("mgmt.web.document-root") == "/www/advUI" then
         SESS["SESS_LAST_PAGE"] = ENV.REQUEST_URI
      end
  end
end
function redirect(url)
    if url == nil then
        url="/"
    end
    local line1=""
	local lcookie=""
    if (string.find(ENV.SERVER_SOFTWARE, "mini_httpd") == nil ) then
        line1 = "HTTP/1.0 302 Found\r\n"
    end
	if ENV.SDB_SETCOOKIE ~= nil and ENV.SDB_SETCOOKIE ~= "" then
		lcookie = string.format("Set-Cookie: %s; Version=1; Discard; Path=/\r\n", ENV.SDB_SETCOOKIE)
	end
    local line2 = string.format("Location: %s\r\n", url)
    local line3 = "Pragma: no-cache\r\n"
    local line4 = "Content-Type: text/html\r\n\r\n"
    local line5 = string.format("<html><meta http-equiv=Refresh content=0;url=%s>\n", url)
    local line6 = "<body></body></html>\n"
    print(line1 .. line2 .. lcookie .. line3 .. line4 .. line5 .. line6)
    --we do need to print everything at once to ensure header is all in one packet.
    os.exit(0)
end

function redirect_lan(url)
    if url == nil then
        url="/"
    end
    --maybe could use ENV.HTTP_HOST here, but sometimes contains "www.google.com"
	local lan_conn = functions.get_path_from_oid_param("ip.primary-lan-oid")
	local lan_addr = string.format("%s.ipaddr", lan_conn)
	local lan_ip = sdb.get(lan_addr)
    local line1=""
    if (string.find(ENV.SERVER_SOFTWARE, "mini_httpd") == nil ) then
        line1 = "HTTP/1.1 302 Found\r\n"
    end
    local lan_port = sdb.get("mgmt.web.http-port")
    if lan_port == "80" then
       line2 = string.format("Location: http://%s%s\r\n", lan_ip, url)
    else
       line2 = string.format("Location: http://%s:%s%s\r\n", lan_ip, lan_port, url)
    end
    local line3 = "Pragma: no-cache\r\n"
	local line4 = "Cache-Control: no-cache\r\n"
    local line5 = "Content-Type: text/html\r\n\r\n"
    local line6 = string.format("<html><body></body></html>")

    print(line1 .. line2 .. line3 .. line4 .. line5 .. line6)
    --we do need to print everything at once to ensure header is all in one packet.
    os.exit(0)
end

function redirect_external(url)
    if url == nil then
        url="/"
    end

    local line1=""
    if (string.find(ENV.SERVER_SOFTWARE, "mini_httpd") == nil ) then
        line1 = "HTTP/1.1 302 Found\r\n"
    end

	local line2 = string.format("Location: http://%s\r\n", url)
    local line3 = "Pragma: no-cache\r\n"
	local line4 = "Cache-Control: no-cache\r\n"
    local line5 = "Content-Type: text/html\r\n\r\n"
    local line6 = string.format("<html><body></body></html>")

    print(line1 .. line2 .. line3 .. line4 .. line5 .. line6)
    --we do need to print everything at once to ensure header is all in one packet.
    os.exit(0)
end

function send_http_code(code)
    local codestr = ""
    if code == "503" then
        codestr = "503 Service Unavailable"
    elseif code == "500" then
        codestr = "500 Internal Error"
    elseif code == "400" then
        codestr = "400 Bad Request"
    elseif code == "404" then
        codestr = "404 Not Found"
    end
    local line1 = string.format("HTTP/1.1 %s\r\n", codestr)
    local line2 = "Connection: close\r\n";
    local line3 = "Content-Type: text/html\r\n\r\n"
    local line4 = string.format("<HTML><HEAD><TITLE>%s</TITLE></HEAD>\n", codestr)
    local line5 = string.format("<BODY>%s</BODY></HTML>\n", codestr)
    print(line1 .. line2 .. line3 .. line4 .. line5)
end

function save_and_redirect()
    functions.sess_save()
    functions.redirect(ENV.SCRIPT_NAME)
end

function save_desired_page()
	local tmp = ENV.REQUEST_HOST .. ENV.SCRIPT_NAME
	local desired_old = sdb.get("mgmt.lan-redirect.desired-page")
	if (desired_old == "") then
		sdb.set("mgmt.lan-redirect.desired-page", tmp)
	end
end

function getgrey(param)
	local disablestr, disableclass
	local val, err = sdb.get(string.format("%s.relevance", param))
	if val == "0" or err == "Item not found"  then
		disablestr = " disabled=\"disabled\""
		disableclass = " class=\"disabled\""
	else
		disablestr = ""
		disableclass = ""
	end
	return disablestr, disableclass
end

function getcheck(param)
	local checkedstr
	if sdb.get(param) == "on" then
		checkedstr = " checked=\"checked\""
	else
		checkedstr = ""
	end
	return checkedstr
end

function get_param_maxlength(param)
  local META = sdb.meta(param)
  local paramtype, var, val
  paramtype =  sdb.aget(param, "type")
  if (paramtype == "number") then
	for var,val in pairs(META) do
		return(string.len(tostring(val)))
	end
  elseif (paramtype == "list" or paramtype == "string") then
	for var,val in pairs(META) do
		return(val)
	end
  end
end

function get_param_type(param)
  local result = sdb.aget(param, "type")
  return(result)
end

function get_param_name(param)
  local result = sdb.aget(param, "name")
  return(result)
end

function set_param_name(param1, param2)
  local result = sdb.aset(param1, "name", param2)
  return(result)
end

function get_path_from_oid_param(a, b)
  local current, METASTR, tp, oid, path, name
  if (b ~= nil) then
      current = b
  else
      current = sdb.get(a)
  end
  if (current == nil or current == "") then
      return ""  -- no match found
  end

  if (sdb.aget(a, "type") == "oid") then
     METASTR = sdb.info("metadata", a)
     tp = {}
     for var, val in ipairs(METASTR) do
       local i = 1
       for word in string.gmatch(val, "%S+") do
        tp[i] = word
        i = i + 1
       end
       oid = tp[1]
       path = tp[2]
       name = tp[3]
       if (current == oid or current == name) then
          if path == nil then
            return ""
          else
            return path
          end
       end
     end
  else
    return ""
  end

  return ""
end
function put_input_radio(displayname, name, val, current, disableOpt, label)
  local labelstart = ""
  local labelend = ""
  local check = ""
  if disableOpt == nil then
    disableOpt=""
  end
  if label ~= nil then
    print(string.format("<input id=\"%s\" type=\"radio\" name=\"" .. name .. "\" value=\"" .. val .. "\"", label))
    labelstart = string.format("<label for=\"%s\">", label)
    labelend = "</label>"
  else
    print("<input type=\"radio\" name=\"" .. name .. "\" value=\"" .. val .. "\"")
  end
  if val == current then
    check = "checked=\"checked\""
  end
  print(string.format(" %s %s />%s_{{%s}}%s", check, disableOpt, labelstart, displayname, labelend))
end

function put_select_option(value, name, selected)
  print("<option value=\"" .. value .. "\"")
  if (value == selected) then
    print(" selected=\"selected\">_{{" .. name .. "}}</option>")
  else
    print(">_{{" .. name .. "}}</option>")
  end
end

function generate_enum_select_options(param, chosen)
	local META
	META = sdb.revenum(param)
	for var, val in orderedPairs(META) do
		functions.put_select_option(val, var, chosen)
	end
end

function generate_oid_select_options(a,b)
	local META
	local current = b
	META = sdb.meta(a)
	--not finished yet
end
function generate_list_select_options(a,b)
	local param = a
	local current = b
	-- not finished yet
end
function generate_select_options(param, chosen)
  local current
  if (chosen ~= nil) then
    current = chosen
  else
    current = sdb.get(param)
  end
  local paramtype = functions.get_param_type(param)
  if (paramtype == "enumeration") then
      functions.generate_enum_select_options(param, current)
  elseif (paramtype == "oid") then
      functions.generate_oid_select_options(param, current)
  else
      functions.generate_list_select_options(param, current)
  end

end

function list_append(a, b)
	if (a == nil or a == "") then
		return b
	else
		return (a .. " " .. b)
	end
end

--[[
    a: old list from where the entries from 'b' should be removed.
    b: list of entries to be removed from 'a'
--]]
function list_remove(a, b)
    local newstring = ""
    if a == nil or a == "" then return "" end
    local ltable = {}
    ltable = functions.split(a, " ")
    for i, val in ipairs(ltable) do
        if b ~= val then
            if newstring == "" then
                newstring = val
            else
                newstring = newstring .. " " .. val
            end
        end
    end
    return newstring
end
function getppplink()
    SESS.SESS_PPPLINK = "notppp"
    local wanconn = get_path_from_oid_param("ip.primary-wan-oid")
    if wanconn == nil then wanconn = "" end
    if sdb.get(string.format("%s.type", wanconn)) == "ppp" then
        local link_oid = sdb.get(string.format("%s.link-oid", wanconn))
        SESS.SESS_PPPLINK = string.format("link[%s]", link_oid)
    end
end

function login(username, password)
    local LOGINLIST
    LOGINLIST = sdb.info("logininfo", username, ENV.SDB_SIDE)
    if password == nil or password == "" then
	   sdb.log(string.format("web login failure for user: %s ", username))
	   return -1
    end
    if password == LOGINLIST[1] then
        if LOGINLIST[3] == "all" or LOGINLIST[3] == "http" or LOGINLIST[3] == "https" then
		   sdb.run("sessupdate", LOGINLIST[2], username)
			sdb.log(string.format("web login success for user: %s ", username))
			return LOGINLIST[2]
		end
	end
	sdb.log(string.format("web login failure for user: %s ", username))
	return -1
end

-- check if new password meets criteria (>8 characters, at least one
--  digit, alpha or punctuation character.
function check_password_criteria(password)
    if string.len(password) < 8 then
        return false
    end
    local alpha = string.find(password, "%a")
    local digit = string.find(password, "%d")
    local punc = string.find(password, "%p")
    if alpha ~= nil then
        if digit ~= nil or punc ~= nil then
            return true
        else
            return false
        end
    else
        if digit ~= nil and punc ~= nil then
            return true
        else
            return false
        end
    end
end

function apply_and_save()
  local res
  local max = 25
  local i = 0
  local isdirty = sdb.get("dirty")
  if SESS.SESS_error_message == nil or
     SESS.SESS_error_message == "" or
     SESS.SESS_error_message == "_{{Changes saved}}" then
    res = sdb.validate()
    if (res ~= 0) then
      SESS.SESS_error_message = ""
      for i, val in ipairs(ERROR) do
       SESS.SESS_error_message = SESS.SESS_error_message .. val .. "<br />"
      end
    else
      res = sdb.apply()
      -- if the sdb is still dirty after an apply, it means the apply
      -- is deferred and we may need to wait before doing the save
      while sdb.get("dirty") == "1" do
          if max == i then break end
          os.execute("sleep 1")
          i = i + 1
      end
      if isdirty == "1" then --only save if something changed
          res = sdb.save()
      end
      if (res == 0) then
        --if the save or apply worked, report changes saved
        SESS.SESS_error_message = "_{{Changes saved}}"
      end
    end
  end
end
function validate_and_save()
  local res
  local max = 25
  local i = 0
  local isdirty = sdb.get("dirty")
  res = sdb.validate()
  if SESS.err == nil then SESS.err = "" end
  if (res ~= 0) then
      SESS.SESS_error_message = ""
      --look through each item in the validation error table
      for key, errormsg in pairs(ERROR) do
          --param_match will be created in the page itself
          --a table of form/sdbparam pairs
          if param_match ~= nil then
              for form, sdbparam in pairs(param_match) do
                  local testform = form .. "{%"
                  if sdbparam == key then
                      errormsg = swallow(errormsg, form)
                      SESS.err = SESS.err .. testform .. errormsg .. "%}"
                      if SESS.SESS_error_message == "" then
                          SESS.SESS_error_message = errormsg .. "<br />"
                      end
                      break
                  end
              end
              --if there isn't a match we still have an error
              if SESS.SESS_error_message == "" then
                  SESS.SESS_error_message = errormsg
              end
          end
      end
  elseif SESS.SESS_error_message == "" or
         SESS.SESS_error_message == nil or
         SESS.SESS_error_message == "_{{Changes saved}}" then
      res = sdb.apply()
      -- if the sdb is still dirty after an apply, it means the apply
      -- is deferred and we may need to wait before doing the save
      while sdb.get("dirty") == "1" do
          if max == i then break end
          os.execute("sleep 1")
          i = i + 1
      end
      if isdirty == "1" then --only save if something changed
          res = sdb.save()
      end
      if (res == 0) then
        --if the save or apply worked, report changes saved
        SESS.SESS_error_message = "_{{Changes saved}}"
      end
  end
end

function start_post_form(url)
 print("<form method=\"post\" action=\"" .. url .. "\">")
 nonce = sdb.run("system.nonce-get")
 print("<input type=\"hidden\" name=\"nonce\" value=\"" .. nonce[1] .. "\" />")
end

function start_multipart_post_form(url)
 print("<form enctype=\"multipart/form-data\" method=\"post\" action=\"" .. url .. "\">")
 nonce = sdb.run("system.nonce-get")
 print("<input type=\"hidden\" name=\"nonce\" value=\"" .. nonce[1] .. "\" />")
end

function check_access()
    if ENV.SDB_SIDE ~= "lan" or ENV.SDB_DOMAIN ~= "1" then
        SESS.SESS_error_message="Permission denied"
        functions.sess_save()
        functions.redirect(ENV.SCRIPT_NAME)
    end
end

function check_wan_access()
	arg = string.format("%s.type", WANCONN)
    wanconn_type = sdb.get(arg)
    SESS_error_message = sdb.set( arg, wanconn_type)
    if SESS_error_message == "Permission denied" then
        functions.sess_save()
        functions.redirect(ENV.SCRIPT_NAME)
    end
end

function pppstatecheck()
if ENV.SCRIPT_NAME ~= SESS.SESS_END_PAGE then
    SESS.SESS_state = "init"
    SESS.SESS_count = 0
    FORM.hidden = ""
elseif SESS.SESS_state == "wait" then
    if sdb.get(string.format("%s.state", WANCONN)) == "up" then
        --check if session is up
        --if so, redirect to final success,
        --if not, continue to refresh
        SESS.SESS_state = "redirect"
    elseif SESS.SESS_count == "3" then
        SESS.SESS_state = "redirect"
        SESS.SESS_error_message = "_{{connection attempt timed out}}"
    else
        SESS.SESS_count = SESS.SESS_count + 1
        FORM.hidden = "refresh"
    end
end
if SESS.SESS_state == "redirect" then
    SESS.SESS_state = "init"
    SESS.SESS_count = 0
    FORM.hidden = ""
    SESS.SESS_IsInitialRedirect = "0"
    functions.sess_save()
    if SESS.SESS_error_message == nil or SESS.SESS_error_message == "" then
        if ENV.SCRIPT_NAME == "/cgi-bin/quickstart.ha" then
            SESS.SESS_RedirectToDiag = "0"
            functions.sess_save()
            functions.redirect("/cgi-bin/home.ha")
        end
    elseif ENV.SCRIPT_NAME == "/cgi-bin/quickstart.ha" and SESS.SESS_RedirectToDiag == "1" then
        SESS.SESS_RedirectToDiag = "0"
        SESS.SESS_error_message = ""
        functions.sess_save()
        functions.redirect("/cgi-bin/diagnostics.ha")
    end
end
end
function savefile(name, command)
  local filename, line1, line2, line3, line4
  -- in this case we are passing the name to be used in the browser Save Dialog
  -- as the first param, and how to generate the data as the second param.
  filename = name
  -- Write HTTP header
  line1 = string.format("Location: %s\r\n", filename)
  line2 = "Pragma: no-cache\r\n"
  line3 = string.format("Content-disposition: attachment; filename=%s\r\n", filename)
  line4 = "Content-Type: text/plain\r\n\r"
  print(line1 .. line2 .. line3 .. line4)
  if command == "logs" then
    for i, val in ipairs(LOG) do print(val) end
  elseif command == "fwlogs" then
    for i, val in ipairs(FIRELOG) do print(val) end
  end
  os.exit()
end

function sbsavefile(name, command)
  local filename, line1, line2, line3, line4
  filename = name
  -- Write HTTP header
  line1 = string.format("Location: %s\r\n", filename)
  line2 = "Pragma: no-cache\r\n"
  line3 = string.format("Content-disposition: attachment; filename=%s\r\n", filename)
  line4 = "Content-Type: text/html\r\n\r"
  print(line1 .. line2 .. line4)
  print("<body><html><pre>")
  if command == "logs" then
    for i = #LOG,1,-1 do
        local val=LOG[i]
        print(val)
    end
  elseif command == "techReadout" then -- For technical readout
    for i, val in ipairs(LOG) do print(val) end
  elseif command == "fwlog" then
    LOG = sdb.run("system.log.firewall.get")
    for i, val in ipairs(LOG) do print(val) end
    repeat
        --> Here 'more' is argument to "sdb.run" to get the next fragment from a large log.
        LOG = sdb.run("system.log.firewall.get", "more")
        if LOG ~= nil and LOG ~= "" then
            for i, val in ipairs(LOG) do print( val)
            end
        end
    until LOG == nil or LOG == ""
  end
  print("</pre></html></body>")
  os.exit()
end

function get_row_button(name)
    local w, index
    SESS.SESS_found = "off"
    for i,v in pairs(FORM) do
		w,index = string.match(i,'([%w]+)_(%d+)')
		if w == name then
		    SESS.SESS_rowoid = index
		    SESS.SESS_found = "on"
		end
    end
end

function extract_oid(s)
	if s == nil then return "" end
	local y, z
	y = 0
	repeat
		z = string.find(s, "[", y+1, true)
		if z ~= nil then y = z end
	until z == nil
	--y now holds the index of the last [
	x = string.find(s, "]", y+1, true)
	return string.sub( string.sub(s, y, x), 2, -2)
end
function getWANPort()
    local PHYENETLIST = sdb.info("objlist", "phy.enet")
    for index, enet in ipairs(PHYENETLIST) do
        local oid = functions.extract_oid(enet)
        local param = enet .. ".port[" .. oid .. "]"
        if sdb.get(string.format("%s.name", param)) == "wan" then
            return param
        end
    end
    return ""
end

function isADSL()
	local cable, err
	if sdb.get("system.board-id") == 0xED then
		return "yes"
	end
	cable, err = sdb.get("phy.dsl.cur-transport")
	if cable == "ATM" then
		return "yes"
	elseif cable == "PTM" then
		return "no"
	elseif err == "Item not found" then
		return "no"
	elseif sdb.get("phy.dsl.transport") == "atm" then
		return "yes"
	else
		return "maybe"
	end
end
function isTransportATM()
	local cable, err
	local boardid = sdb.get("system.board-id")
	local sku = sdb.get("system.sku")
	local wanport = getWANPort()
	local waneth = string.sub(wanport, 1, string.find(wanport, "%]"))
	if boardid == "0xED" then
		return "yes"
	elseif boardid == "0xF8" and sku == "70" then
		return "no"
	elseif boardid == "0xF8" and
		   sku == "1" and
		   sdb.get("phy.dsl.cur-transport") == "Not connected" and
		   sdb.get(string.format("%s.state", waneth)) == "up" then
		   --DSL isn't plugged in, but Ethernet WAN is
		return "no"
	end
	cable, err = sdb.get("phy.dsl.cur-transport")
	if cable == "ATM" then
		return "yes"
	elseif cable == "PTM" then
		return "no"
	elseif cable == "PTM BOND" then
		return "no"
	elseif err == "Item not found" then
		return "no"
	elseif sdb.get("phy.dsl.transport") == "atm" then
		return "yes"
	elseif sdb.get("phy.dsl.transport") == "ptm" then
		return "no"
    elseif sku == "64" and sdb.get("system.sku-64.cur-mode") == "adsl" then
        return "yes"
    else
		return "maybe"
	end
end
function findethlink(vcnum)
	local target, OBJLIST, value, err
	target = string.format("vc-%d", vcnum)
	OBJLIST = sdb.info("objlist", "link")
	for i, val in ipairs(OBJLIST) do
		value, err = sdb.get(string.format("%s.port-vlan.ports", val))
		if err ~= "Item not found" then
			if string.find(value, target, 1, true) ~= nil then
				return functions.extract_oid(val)
			end
		end
		value, err = sdb.get(string.format("%s.tagged-vlan.ports", val))
		if err ~= "Item not found" then
			if string.find(value, target, 1, true) ~= nil then
				return functions.extract_oid(val)
			end
		end
	end
	return "0"
end
function findppplink(conn)
	local mylink = sdb.get(string.format("conn[%s].link-oid", conn))
	if mylink == "PPPoE" then
	    MYLIST = sdb.info("objlist", "link")
	    for i, val in ipairs(MYLIST) do
	        local myname = sdb.aget(val, "name")
	        if myname == "PPPoE" then return functions.extract_oid(val) end
	    end
	else
	    return mylink
	end
end
function isSIPALGSupported()
    local model = sdb.get("system.model-name")
    if model == "NM55" then
        return false
    else
        return true
    end
end
function isESPALGSupported()
    local model = sdb.get("system.model-name")
    if model == "NM55" then
        return false
    else
        return true
    end
end
function findconn(vcnum)
	local ethlink, ethname, OBJLIST, mytype, value, linkoid, sub
	--find conn by oid number
	ethlink = findethlink(vcnum)
	ethname = sdb.aget(string.format("link[%s]", ethlink), "name")
	OBJLIST = sdb.info("objlist", "conn")
	for i, val in ipairs(OBJLIST) do
		mytype = sdb.get(string.format("%s.type", val))
		if mytype ~= "ppp" then
			value = sdb.get(string.format("%s.link-oid", val))
			if value == ethlink or value == ethname then
				return functions.extract_oid(val)
			end
		else
			linkoid = sdb.get(string.format("%s.link-oid", val))
			sub = sdb.get(string.format("link[%s].ppp-lcp.sub-link-oid", linkoid))
			if sub == ethlink or sub == ethname then
				return functions.extract_oid(val)
			end
		end
	end
	return "0"
end
function findconnandlink(vcnum)
	local conn, link
	conn = findconn(vcnum)
	if sdb.get(string.format("conn[%s].type", conn)) == "ppp" then
		link = findppplink(conn)
	else
		link = findethlink(vcnum)
	end
	return conn, link
end
function getppplink()
    local link_oid
    SESS.SESS_PPPLINK = "notppp"
    local wanconn = get_path_from_oid_param("ip.primary-wan-oid")
    if wanconn == nil then wanconn = "" end
    if sdb.get(string.format("%s.type", wanconn)) == "ppp" then
        link_oid = sdb.get(string.format("%s.link-oid", wanconn))
        SESS.SESS_PPPLINK = string.format("link[%s]", link_oid)
    end
end
function isWanConn()
    local wanconn= functions.get_path_from_oid_param("ip.primary-wan-oid")
    if wanconn ~= nil and wanconn ~="" then
       return true
    end
    return false
end
function isRemoteAccess()
    local wanconn= functions.get_path_from_oid_param("ip.primary-wan-oid")
    if wanconn ~= nil and wanconn ~="" then
       local cmd=string.format("%s.state",wanconn)
       if sdb.get(cmd) == "up" then
          return true
       end
    end
    return false
end
function isDslEnabled()
    local enabled = sdb.get("phy.dsl.enable")
    if enabled ~= "" and enabled == "on" then
       return true
    end
    return false
end
function isDualEth()
  local enetmax, param
  enetmax = sdb.info("objmax", "phy.enet")
	for index = 1, enetmax[1] do
		param = string.format("phy.enet[%d].port.name", index)
		if string.find(sdb.get(param), "wan") ~= nil then
			return true
		end
	end
	return false
end
function get_connlist()
	local connlist, conn_name
	connlist = sdb.info("objlist", "conn")
	for i, conn in ipairs(connlist) do
		conn_name = sdb.aget(conn, "name")
		if conn_name ~= "" then
			connlist[i] = conn_name
		end
	end
	return connlist
end
function isVoiceSupported()
    voiceStatus = sdb.get("voice.enabled")
    if nil ~= voiceStatus and "1" == voiceStatus then
        return true
    end
    return false
end
function isWirelessSupported()
    local WOBJLIST
    WOBJLIST = sdb.info("objlist", "phy.wl80211")
    for i, val in ipairs(WOBJLIST) do
		if string.find(val, "phy.wl80211") ~= nil then
		    return true
	    end
	end
	return false
end
function isGuestSSIDSupported()
    return false
end
function isWirelessDualBandSupported()
    local DBOBJLIST
    DBOBJLIST = sdb.meta("phy.wl80211.band-lock")
    for var, val in pairs(DBOBJLIST) do
    	if var == "5.0" then
        	return true
        end
    end
    return false
end
function isDualRadio()
  local WOBJLIST
  WOBJLIST = sdb.info("objlist", "phy.wl80211")
  wintcount = 0
  for i, val in ipairs(WOBJLIST) do
      wintcount = wintcount + 1
  end
  if wintcount > 1 then
      return true
  else
      return false
  end
end
function isHPNASupported()
    local HOBJLIST
    HOBJLIST = sdb.info("objlist", "phy.hpna")
    for i, val in ipairs(HOBJLIST) do
		if val == "phy.hpna" then
		    return true
	    end
	end
	return false
end
function isBatterySupported()
  local batval, baterr = sdb.get("system.battery.enable")
  local mname = sdb.get("system.model-name")
  if (mname == "NVG589" or mname == "NVG510" or mname == "NVG599") and baterr ~= "Item not found" then
      return true
  else
      return false
  end
end

function get_num_voice_ports()
	local maxLines = sdb.get("voice.max-fxs-ports")
	if false == functions.isVoiceSupported() or nil == maxLines or "" == maxLines then
		maxLines = "0"
	end
	return tonumber(maxLines)
end

-- function checks whether voice is enabled or not. If found on any one port return true.
function voice_ports_enabled()
	local fxs_ports = functions.get_num_voice_ports()
	for i = 1, fxs_ports do
	local portEnabled = sdb.get("voice.phone["..i.."].enabled")
	    if nil ~= portEnabled and "on" == portEnabled then
		return true
	    end
	end
	return false
end

function isSambaSupported()
    local data, err
    if isUsbSupported() == false then
        return false
    end
    data, err = sdb.get("samba.enable")
	if err == "Item not found" then
		return false
	end
	return true
end

function isUsbSupported()
    local data, err
    data, err = sdb.get("system.usbhost.dev-type")
	if err == "Item not found" then
		return false
	else
		return true
	end
end

function isDlnaSupported()
	local data, err
	data, err = sdb.get("mgmt.dlna.enable")
	if err == "Item not found" then
		return false
	else
		return true
	end
end

function isBonjourSupported()
    local data, err
    data, err = sdb.get("ip.bonjour.enable")
	if err == "Item not found" then
		return false
	else
		return true
	end
end

function IsWiFiSupported()
    local WOBJLIST
    WOBJLIST = sdb.info("objlist", "phy.wl80211")
    for i, val in ipairs(WOBJLIST) do
        if val == "phy.wl80211" then
            return true
         end
    end
    return false
end

-- We only support letting the user easily configure the box as a
-- bridge for ADSL generic units, so no NVG589.  We really
-- doubt a user will want to turn an expensive NVG589 into a bridge ...
function IsBridgeModeSupported()
    local mname = sdb.get("system.model-name")
    -- Only supported on certain hardware
    if mname == "2247-N8" or mname == "NVG510" then
        return true
    else
        return false
    end
end

function IsFlashNand()
    local f = io.popen("cat /proc/mtd")
    for line in f:lines() do
        if string.find(line, "rootfs_update") then
           io.close(f)
           return true
         end
    end
    io.close(f)
    return false
end

-- used to get every other row with blue background
-- <% cls="b" %> ... <tr class="<% functions.FlipRowsBgrndColor() %>">
function FlipRowsBgrndColor()
  local cls
  if cls == "a" then
      cls = "b"
  else
      cls = "a"
  end
  print(cls)
end
function get_row_button2D(name)
    local w, rowindex, colindex
    SESS.SESS_found = "off"
    for i,v in pairs(FORM) do
      w,rowindex,colindex = string.match(i,'([%w]+)_(%d+)_(%d+)')
      if w == name then
        SESS.SESS_rowindex = rowindex
        SESS.SESS_colindex = colindex
        SESS.SESS_found = "on"
      end
    end
end
function RES_ApplySave()
   ui = sdb.get("mgmt.web.document-root")
   if string.match(ui,"residential") ~= nil then
           apply_and_save()
   end
end
function fixchannel(button)
	local oldtype, newtype
	local oldstandard = sdb.get("phy.wl80211.standard")
	local auto = sdb.get("phy.wl80211.autochannel")
	if (oldstandard == "an") or (oldstandard == "a-only") or (oldstandard == "n-only") then
		oldtype = "a"
	else
		oldtype = "b"
	end
	if (FORM.standard == "an") or (FORM.standard == "a-only") or (FORM.standard == "n-only") then
		newtype = "a"
	else
		newtype = "b"
	end

	--if changing standard if auto on, we may need to change channel
	if (auto == "on") and (button == nil) then
		if oldtype ~= newtype then
			if newtype == "a" then
				sdb.set("phy.wl80211.channel", "36")
			else
				sdb.set("phy.wl80211.channel", "1")
			end
		end
	--if changing to auto we may need to change channel
	elseif FORM.channelplusauto == "0" and auto == "off" then
		if newtype == "a" then
			sdb.set("phy.wl80211.channel", "36")
		else
			sdb.set("phy.wl80211.channel", "1")
		end
	end
end

function printlang()
    local lang = sdb.get("mgmt.web.language")
    if lang == "sb_de" then
        print("de")
    elseif lang == "sb_it" then
        print("it")
    elseif lang == "sb_fr" then
        print("fr")
    else
        print("en")
    end
end

function has_supplicant()
	local active = sdb.get("system.supplicant.active")
	if active == nil or active == "false" then
		return false
	else
		return true
	end
end

function get_supp_link_path()
	local path = nil
	if functions.has_supplicant() == false then
		return nil
	else
		local oid = sdb.get("system.supplicant.link-oid")
		if oid == nil then
			return nil
		else 
			path = string.format("link[%s].supplicant", oid)
			return path
		end
	end
end

function bool_to_string(val)
	if val then
		return "true"
	else
		return "false"
	end
end

function get_supplicant_authenticated()
	local path = get_supp_link_path()
	if path == nil then
		return false
	else
		local eapol_state = sdb.get("system.supplicant.peer-state")
		if eapol_state == nil then
			return false
		elseif eapol_state == "AUTHENTICATED" then
			return true
		else
			return false
		end
	end
end
	
function isNTP()
    if sdb.get("ip.ntp.date") == "unknown" then
        return false
    else
        return true
    end
end
function isWANUP()
    local wanconn = functions.get_path_from_oid_param("ip.primary-wan-oid")
    local wanip = sdb.get(string.format("%s.ipaddr", wanconn))
    if (wanip == nil or wanip == "" or wanip == "0.0.0.0") or sdb.get(string.format("%s.state", wanconn)) ~= "up" then
        return false
    else
        return true
    end
end
function bits_to_mask(bits)
 local mapping_table = { ["30"] = "255.255.255.252", ["28"] = "255.255.255.240", ["26"] = "255.255.255.192" ,
                            ["24"] = "255.255.255.0", ["22"] = "255.255.252.0", ["20"] = "255.255.240.0" ,
                            ["18"] = "255.255.192.0", ["16"] = "255.255.0.0", ["14"] = "255.252.0.0" ,
                            ["12"] = "255.240.0.0", ["10"] = "255.192.0.0", ["8"] = "255.0.0.0" ,
                            ["6"] = "252.0.0.0", ["4"] = "240.0.0.0", ["2"] = "192.0.0.0" }
     for i,val in pairs(mapping_table) do
         if i == bits then return (val) end
     end
     return ("255.255.255.255")
end

function showroutes(type)
    local f, i, network, bit, netmask, a, b
    --Get main route table info
    local tbl = {}
    local a
    local j = 97
    local c = "<tr><td class=\"col2\" align=\"left\">%s</td><td class=\"col2\" align=\"left\">%s</td><td class=\"col2\" align=\"left\">%s</td><td class=\"col2\" align=\"left\">%s</td><td class=\"col2\" align=\"left\">%s</td><td class=\"col2\" align=\"left\">%s</td><td class=\"col2\" align=\"left\">%s</td><td class=\"col2\" align=\"left\">%s</td><td class=\"col2\" align=\"left\">%s</td></tr>"
    f = io.popen(string.format("ip route show table %s", type), "r")
    for line in f:lines() do
        i = 1
        local data = {}
        for word in string.gmatch(line, "%S+") do
            data[i] = word
            i = i + 1
        end
        if data[2] == "dev" then
            network,bit = string.match(data[1],'([%d%.]+)[/]*([%d%.]*)')
            if bit ~=  nil then
                netmask = functions.bits_to_mask(bit)
            end
            if network ~=  nil and netmask ~=  nil and data[9] ~= nil and data[3] ~= nil  then
            a = string.find(network, "%.")
            if a ~= nil and a > 1 then
                b=string.format(c, string.format("322%s",string.char(j)),network, netmask, "-",data[9],"0","-", data[3],"-")
                tbl[ network .. "," .. netmask ] = b
            end
			end
        elseif data[1] == "default" then
		if data[3] ~= nil and data[5] ~= nil  then
            b=string.format(c, string.format("322%s",string.char(j)),"0.0.0.0", "0.0.0.0","-", data[3],"0","-", data[5],"-")
            tbl[ "0.0.0.0" .. "," .. "0.0.0.0" ] = b
		end
        elseif data[2] == "via" then
            network,bit = string.match(data[1],'([%d%.]+)[/]*([%d%.]*)')
            if bit ~=  nil then
               netmask = functions.bits_to_mask(bit)
            end
            if network ~=  nil and netmask ~=  nil and data[3] ~= nil and data[5] ~= nil  then
            b=string.format(c, string.format("322%s",string.char(j)),network, netmask,"-", data[3],"0","-", data[5],"-")
            tbl[ network .. "," .. netmask ] = b
			end
        elseif (data[1] == 'local' or data[1] == 'broadcast') then
            network,bit = string.match(data[2],'([%d%.]+)[/]*([%d%.]*)')
            if bit ~=  nil then
               netmask = functions.bits_to_mask(bit)
            end
            if type == "local" then
            if network ~=  nil and netmask ~=  nil and data[10] ~= nil and data[4] ~= nil  then
                b=string.format(c, string.format("322%s",string.char(j)),network, netmask,"-", data[10],"0","-", data[4],"-")
			end
            else
            if network ~=  nil and netmask ~=  nil and data[12] ~= nil and data[4] ~= nil  then
                b=string.format(c,string.format("322%s",string.char(j)), network, netmask,"-", data[12],"0","-", data[4],"-")
			end
            end
            tbl[ network .. "," .. netmask ] = b
        end
j = j + 1
    end
    io.close(f)
    for subnet, line in functions.orderedPairs(tbl) do
        print(line)
    end
end
function display_all_routes(type)
    local f, i, network, bit, netmask, a, b
    --Get main route table info
    local tbl = {}
    local a
    local c = "<tr><td class=\"col2\">%s</td><td class=\"col2\">%s</td><td class=\"col2\">%s</td><td class=\"col2\">%s</td></tr>"
    f = io.popen(string.format("ip %s", type), "r")
    for line in f:lines() do
        i = 1
        local data = {}
        for word in string.gmatch(line, "%S+") do
            data[i] = word
            i = i + 1
        end
        if data[2] == "dev" then
            network,bit = string.match(data[1],'([%d%.]+)[/]*([%d%.]*)')
            if bit ~=  nil then
                netmask = functions.bits_to_mask(bit)
            end
            if network ~=  nil and netmask ~=  nil and data[9] ~= nil and data[3] ~= nil  then
				a = string.find(network, "%.")
				if a ~= nil and a > 1 then
				    b=string.format(c, network, netmask, data[9], data[3])
				    tbl[ network .. "," .. netmask ] = b
				end
            end
        elseif data[1] == "default" then
		if data[3] ~= nil and data[5] ~= nil  then
            b=string.format(c, "0.0.0.0", "0.0.0.0", data[3], data[5])
            tbl[ "0.0.0.0" .. "," .. "0.0.0.0" ] = b
		end
        elseif data[2] == "via" then
            network,bit = string.match(data[1],'([%d%.]+)[/]*([%d%.]*)')
            if bit ~=  nil then
               netmask = functions.bits_to_mask(bit)
            end
            if network ~=  nil and netmask ~=  nil and data[3] ~= nil and data[5] ~= nil  then
            b=string.format(c, network, netmask, data[3], data[5])
            tbl[ network .. "," .. netmask ] = b
			end
        elseif (data[1] == 'local' or data[1] == 'broadcast') then
            network,bit = string.match(data[2],'([%d%.]+)[/]*([%d%.]*)')
            if bit ~=  nil then
               netmask = functions.bits_to_mask(bit)
            end
            if type == "local" then
            if network ~=  nil and netmask ~=  nil and data[10] ~= nil and data[4] ~= nil  then
                b=string.format(c, network, netmask, data[10], data[4])
			end
            else
            if network ~=  nil and netmask ~=  nil and data[12] ~= nil and data[4] ~= nil  then
                b=string.format(c, network, netmask, data[12], data[4])
			end
            end
            tbl[ network .. "," .. netmask ] = b
        end
    end
    io.close(f)
    for subnet, line in functions.orderedPairs(tbl) do
        print(line)
    end
end

function get_lhd(scan)
  local NEXTLIST
  if scan == true then
      sdb.run("ip.hostdb.scan")
      os.execute("sleep 3")
  end
  RUNLIST = sdb.run("ip.hostdb.get")
  if RUNLIST ~= nil then
    while NEXTLIST == nil or #NEXTLIST > 0 do
        NEXTLIST = sdb.run("ip.hostdb.getnext")
        if NEXTLIST ~= nil then
            for i, val in ipairs(NEXTLIST) do
                table.insert(RUNLIST,val)
            end
        end
    end
  end
  return RUNLIST
end
function get_dns()
    local primary = sdb.get("ip.dns.curr-address1")
    local secondary = sdb.get("ip.dns.curr-address2")
    SESS.SESS_PRIM_NAME = ""
    SESS.SESS_SEC_NAME = ""
    local wantype = string.format("%s.type", WANCONN)
    if sdb.get(wantype) ~= "static"  then
        if primary ~= nil and primary ~= "" and primary ~= "0.0.0.0" then
            host = sdb.run("ip.dns.lookup", primary)
            if host[1] ~= nil and host[1] ~= "Name or service not known" then
                SESS.SESS_PRIM_NAME = host[1]
            end
        end
        if secondary ~= nil and secondary ~= "" and secondary ~= "0.0.0.0" then
            host = sdb.run("ip.dns.lookup", secondary)
            if host[1] ~= nil and host[1] ~= "Name or service not known" then
                SESS.SESS_SEC_NAME = host[1]
            end
        end
    end
    return primary, secondary
end
function colorclass(var)
   if var == "Pass" then
       print("class=\"respass\"")
   elseif var == "Fail" then
       print("class=\"resfail\"")
   end
end
function isIPv6Enabled()
    local data, err
    data, err = sdb.get("ip6.enable")
	if data == "on" then return true end
    return false
end

function name2ruleset(matchname)
    --from the application name get the ruleset number
    for i = 1, 256 do
        if matchname == sdb.aget(string.format("app-hosting.application[%d]", i), "name") then
            return i
        end
    end
    return 0
end

function is_app_used(oid)
    --get the name of the app from the oid passed in
    local matchname = sdb.aget(string.format("app-hosting.application[%s]", oid), "name")
    --get the list of hosted servers
    local SERVLIST = sdb.info("objlist", "app-hosting.server")
    if SERVLIST[1] == nil then return false end
    for i, val in ipairs(SERVLIST) do
        local appname = sdb.get(string.format("%s.application", val))
        if matchname == appname then
            return true
        end
    end
    return false
end

function appname_to_lhdname(appname)
    if (appname == nil) then
      return "_{{unknown}}"
    end
    local temp1 = string.sub(appname, string.find(appname, ":") + 1)
    appname = string.sub(temp1, string.find(temp1, ":")+1) -- from 2nd occurrance of : + 1 to end of string
    -- If no MAC address, use the IP address
    if appname == "" then
       appname = string.gsub(temp1, ":", "")
    end
    --check lhd to match the host info (mac or IP)
    LHDLIST = get_lhd(false)
      if LHDLIST ~= nil then
        for i, val in ipairs(LHDLIST) do
            if string.find(val, appname) ~= nil then
                --return the lhd name for that mac
                local temp2 = string.sub(val, string.find(val, "|") + 1)
                local temp3 = string.sub(temp2, string.find(temp2, "|") + 1)
                if string.find(temp3, "unknown") then
                    return string.sub(temp2, 1, string.find(temp2, "|")-1)
                else
                    return string.sub(temp3, 1, string.find(temp3, "|")-1)
                end
            end
        end
    end
    return "_{{unknown}}"
end

function build_sb_device_list(chosen, activeonly)
    -- // Populate the drop down box with the device names.
    local SBRUNLIST = functions.get_lhd(false)
    local tbl = {}
    local b
    local c = "<option value=\"%s\" %s>%s</option>"
    local ipmac = sdb.get("ip.wan-allocation.passthrough.cur-host-mac-addr")

    for i, val in ipairs (SBRUNLIST) do
        --1. index 2. ip addr 3. name 4. mac 5. on/off 6. ether/wire 7. static/dhcp
        iptable = {}
        iptable = functions.split(val, "|")
        if (iptable[2] ~= nil and activeonly == nil) or
        (iptable[2] ~= nil and iptable[5] == "on" and activeonly == "1" and
           ipmac ~= iptable[4]) then
            if chosen == iptable[3] then
                sel = "selected=\"selected\""
            else
                sel = ""
            end
            if string.find(iptable[3], "unknown", 1, true) ~= nil then
                b=string.format(c, iptable[3], sel, iptable[2])
            else
                b=string.format(c, functions.string_to_html(iptable[3]), sel, functions.string_to_html(iptable[3]))
            end
            tbl[ iptable[2] .. "," .. iptable[4] ] = b
        end
    end
    for ipMac, line in functions.orderedPairs(tbl) do
      print(line)
    end
end

-- from http://lua-users.org/wiki/SplitJoin
function functions:split(sep)
        local sep, fields = sep or ":", {}
        local pattern = string.format("([^%s]+)", sep)
        self:gsub(pattern, function(c) fields[#fields+1] = c end)
        return fields
end
function check_form_vars()
	for var, val in pairs(FORM) do
		if string.find(val, "<script") ~= nil then
			functions.redirect(ENV.SCRIPT_NAME)
		end
	end
end
function get_portstring_by_prot(tab, prot, compflg, compstr)
    local mystr = ""
    local joinstr = ""
    for i, val in ipairs(tab) do
        local myprot = sdb.get(string.format("%s.protocol", val))
        local extminport = sdb.get(string.format("%s.ext-min-port", val))
        local extmaxport = sdb.get(string.format("%s.ext-max-port", val))
        if prot == myprot then
            local newstr, checkstr1, checkstr2
            if extminport == extmaxport then
                newstr = extminport
                checkstr1 = extminport .. "%,"
                checkstr2 = extminport
            else
                newstr = extminport .. "-" .. extmaxport
                -- when using gsub, chars like - and , need the % escape char
                checkstr1 = extminport .. "%-" .. extmaxport .. "%,"
                checkstr2 = extminport .. "%-" .. extmaxport
            end
            if compflg == true then -- doing the second list (udp)
                loc1 = string.find(compstr, newstr, 1, true)
                if loc1 ~= nil then
                    joinstr = joinstr .. newstr .. ","
                    -- first eliminate the dup if it has a comma
                    compstr = string.gsub(compstr, checkstr1, "")
                    -- next eliminate the dup if it is at the end, no comma
                    compstr = string.gsub(compstr, checkstr2, "")
                else
                    mystr = mystr .. newstr .. ","
                end
            else -- when doing the 1st list (tcp)
                mystr = mystr .. newstr .. ","
            end
        end
    end
    return mystr, compstr, joinstr
end
function get_portstring(appname)
    tcpstr = ""
    udpstr = ""
    local portstring = ""
    if string.byte(appname) == 42 then
        appname = string.sub(appname, appname + 1)
    end
    local oid = functions.name2ruleset(appname)
    local ruleset = sdb.get(string.format("app-hosting.application[%d].ruleset-oid", oid))
    RULELIST = sdb.info("objlist", string.format("app-hosting.ruleset[%s].rules", ruleset))
    tcpstr = get_portstring_by_prot(RULELIST, "tcp", false )
    udpstr = get_portstring_by_prot(RULELIST, "udp", false)
    bothstr = get_portstring_by_prot(RULELIST, "both", false)
    return udpstr, tcpstr, bothstr
end

--add_tags is called by UIs that need to translate.
--this way UIs like att don't need to bother
--add_tags work for both set and apply errors
--apply errors can have multiple lines
function add_tags(str)
  local msg = ""
  local i
  local s
  local begin_tag = "_{{"
  local end_tag = "}}"
  local table = {}
  table = functions.split(str, "\n")
  for i, s in ipairs (table) do
    s = string.gsub(s, "<br />", "")
    msg = msg .. string.format("_{{%s}}<br />", s);
  end
  return msg
end

function ltrim(str)
  return string.gsub(str,"^%s*", "")
end

function routeSplit(str)
	vals = {}
	endPos = string.find(str,'%s')
	vals[1] = string.sub(str,1,endPos-1)

	local valindex = 2
	repeat
	 	str = string.sub(str,endPos)
		str = ltrim(str)

		starFound = string.find(str,'^[%*]')

		if nil ~= starFound then
			if valindex == 2 then
				vals[2] = ""
			    valindex = valindex + 1
			end
			vals[valindex] = "*"
			endPos = 2
		else
			icmpErrFound = string.find(str,'^[!][AHNPSFX]')
			if nil ~= icmpErrFound then --Got an ICMP Error
				valindex = valindex - 1
				vals[valindex] = vals[valindex].." "..string.sub(str,1,icmpErrFound+1)
	            endPos = icmpErrFound + 2
			else
				endPos = string.find(str,'%d*%.%d*%sms[^%S]*')
				if nil ~= endPos then --Got RTT in ms
				    startPos = 1
					if endPos > 1 then     --Got the domain name/IP
						if vals[2] == nil or vals[2] == "" then
							vals[2] = string.sub(str,1,endPos-1)
							if valindex == 2 then
							    valindex = 3
							end
						end
						startPos = endPos
					end
					endPos = string.find(str,'ms',endPos,true)
					if nil ~= endPos and endPos > startPos then
				    	vals[valindex] = string.sub(str,startPos,endPos+1)
				    end
	                endPos = endPos + 2
				end
			end
		end
		valindex = valindex + 1
	until str == ""
	return vals
end

function IsPhoneRegistered(line)
	regStat = sdb.get("voice.phone["..line.."].reg-state")
	if nil ~= regStat and regStat == "Registered" then
  		return "_{{Yes}}"
  	else
  		return "_{{No}}"
  	end
end

function getCallState(line)
	callState = sdb.get("voice.phone["..line.."].call[1].call-state")
	if nil == callState or "Idle" == callState then
		callState = sdb.get("voice.phone["..line.."].call[2].call-state")
		if nil ~= callState and "Idle" ~= callState then
			return "_{{Call}}"
		else
			return "_{{Idle}}"
		end
	else
		return "_{{Call}}"
  	end
end

function getPhoneNumber(line)
	local formatedNum,phNum = "",""
	local pos = {2,5,9,12,100} --positions from back where space to be inserted
--	local sipuri = sdb.get("voice.phone["..line.."].sip-uri")

--	if nil ~= sipuri and "" ~= sipuri then
--		local beg,fin
--		beg, fin = string.find(sipuri, "[%+]?%d+")
--		if nil ~= beg and nil ~= fin then
--			phNum = string.sub(sipuri,beg,fin)
--		end
--	end

	phNum = sdb.get("voice.phone["..line.."].phone-number")
	if nil ~= phNum then
 		len = string.len(phNum)
 		if nil ~= len and 0 < len then
 			i = 1
			formatedNum = phNum
			while (len - pos[i]) > 0 do
				formatedNum = string.sub(formatedNum,1,len-pos[i]).." "..string.sub(formatedNum,len-pos[i]+1)
				i = i+1
				len = len+1
			end
		end
	end
  	return formatedNum
end

function getCallCount(line)
	callCnt = sdb.get("voice.phone["..line.."].total-calls")
	if nil ~= callCnt and "" ~= callCnt then
  		return callCnt
  	else
  		return "0"
  	end
end

function getRegRetryCount(line)
	retryCnt = sdb.get("voice.phone["..line.."].reg-retry-cnt")
	if nil ~= retryCnt and "" ~= retryCnt then
  		return retryCnt
  	else
  		return "0"
  	end
end

function get_pubsub()
    local CONNLIST = sdb.info("objlist", "conn")
    local puboid = ""
    for i, val in pairs(CONNLIST) do
        if string.find(sdb.get(string.format("%s.lan-type", val)), "public") ~= nil then
            puboid = functions.extract_oid(val)
        end
    end
    return puboid
end

function check_install_state(nexturl)
  if SESS.SESS_FwInstallRunning == "1" then
    local state = sdb.get("system.firmware.install-state")
    if state == "Image validated" then
       SESS.SESS_Restarting = "1"
       SESS.SESS_FirmwareInstalled = "1"
       SESS.SESS_installing = "1"
       SESS.SESS_HIDDEN = "0"
       functions.sess_save()
       functions.redirect(nexturl)
    elseif state == "Invalid image" then
      SESS.SESS_error_message = "Firmware image is invalid"
      SESS.SESS_FwInstallRunning = "0"
      SESS.SESS_HIDDEN = ""
    -- the case below might happen in server-based upgrade (e.g. Swisscom)
    elseif state == "Image not found" then
      SESS.SESS_error_message = "File Update Failed!"
      SESS.SESS_FwInstallRunning = "0"
      SESS.SESS_HIDDEN = ""
    elseif state == "Install cancelled" then
       SESS.SESS_error_message = "Firmware installation was canceled due to a phone call in progress"
       SESS.SESS_FwInstallRunning = "0"
       SESS.SESS_HIDDEN = ""
    elseif state == "Download cancelled" then
       SESS.SESS_error_message = "Firmware download canceled since battery is discharging"
       SESS.SESS_FwInstallRunning = "0"
       SESS.SESS_HIDDEN = ""
    elseif state == "Install delayed" then
       SESS.SESS_error_message = "Firmware installation was delayed due to a phone call in progress"
    else
       -- for a web install, we need to wait for one of the above errors.
       -- don't bail on any of the below status items
       -- "Idle" "Downloading" or "Image not found"
       sdb.set("system.firmware.web-state", "Installing")
       SESS.SESS_error_message = ""
    end
    functions.sess_save()
  end
end

function string_to_html(mystring)
    if mystring ~= "" and mystring ~= nil then
        -- the order is important, & must be first, # must be before /
        mystring = string.gsub(mystring, "&", "&amp;")
        mystring = string.gsub(mystring, "\"", "&quot;")
        mystring = string.gsub(mystring, "<", "&lt;")
        mystring = string.gsub(mystring, ">", "&gt;")
        mystring = string.gsub(mystring, "#", "&#35;")
        mystring = string.gsub(mystring, "\/", "&#47;")
        mystring = string.gsub(mystring, "\´", "&acute;")
    end
    return mystring
end

function html_to_string(mystring)
    if mystring ~= "" and mystring ~= nil then
        --the order is important, & must be last
        mystring = string.gsub(mystring, "&quot;", "\"")
        mystring = string.gsub(mystring, "&lt;", "<")
        mystring = string.gsub(mystring, "&#47;", "\/")
        mystring = string.gsub(mystring, "&gt;", ">")
        mystring = string.gsub(mystring, "&#35;", "#")
        mystring = string.gsub(mystring, "&acute;", "\´")
        mystring = string.gsub(mystring, "&amp;", "&")
    end
    return mystring
end

function ipv6ConnIsTypeRdCpe()
  local rdType = ""
  local ip6wanconn = ""
  local type = ""
  if sdb.get("ip6.enable") == "on" then
    ip6wanconn = sdb.get("ip6.primary-wan-oid")
    if ip6wanconn ~= nil and  ip6WanConn ~= "" then
      type = sdb.get(string.format("ip6.conn[%s].type", ip6wanconn))
    end
    if type == "rd" then
      rdType = sdb.get(string.format("ip6.conn[%s].rd.type", ip6wanconn))
    end
  end
  if rdType == "cpe" then
    return "1"
  end
  return "0"
end

function full_diag()
	 -- set up default strings
	local lan_conn = sdb.get("ip.primary-lan-oid")
	local wan_conn = sdb.get("ip.primary-wan-oid")
	local skippedstr = "Skipped"
	local passstr = "Pass"
	local failstr = "Fail"
	local sku = sdb.get("system.sku")
	-- might need LAN and WAN session variables
	SESS.SESS_EtherResult = skippedstr
	SESS.SESS_BridgeResult = skippedstr
	SESS.SESS_DSLResult   = skippedstr
	SESS.SESS_DSLSignalResult   = skippedstr
	SESS.SESS_ATMResult   = skippedstr
	SESS.SESS_AtmSignalResult = skippedstr
	SESS.SESS_AtmOamSegPingResult = skippedstr
	SESS.SESS_AtmOamEndPingResult = skippedstr
	SESS.SESS_PTMResult   = skippedstr
	SESS.SESS_PPPoEResult = skippedstr
	SESS.SESS_PPPEtherSublinkResult = skippedstr
	SESS.SESS_PPPConnectionResult = skippedstr
	SESS.SESS_AuthResult  = skippedstr
	SESS.SESS_IPResult    = skippedstr
	SESS.SESS_IPPPPResult    = skippedstr
	SESS.SESS_IPEthernetResult = skippedstr
	SESS.SESS_ValidWanIpResult = skippedstr
	SESS.SESS_PingDfGatewayResult = skippedstr
	SESS.SESS_DNSResult   = skippedstr
	SESS.SESS_DNSRevLookupPriResult = skippedstr
	SESS.SESS_DNSRevLookupSecResult = skippedstr
	SESS.SESS_SixRdResult = skippedstr
	SESS.SESS_pingdfhost = skippedstr

	if lan_conn ~= nil and lan_conn ~= "" then
		if diag.BridgeIsWorking(lan_conn) ~= false then
			SESS.SESS_BridgeResult = passstr
		else
			SESS.SESS_BridgeResult = failstr
		end
		if diag.BridgeIsWorking(lan_conn) ~= false and
			diag.EtherIsWorking(lan_conn) ~= false and
			diag.IPIsWorking(lan_conn) ~= false then
			SESS.SESS_EtherResult = passstr
		else
			SESS.SESS_EtherResult = failstr
		end
	end

	if diag.IsDslSupported() ~= false then
		if diag.DSLIsWorking() ~= false then
			SESS.SESS_DSLResult = passstr
			SESS.SESS_DSLSignalResult = passstr
		else
			if diag.DSLHasSignal() ~= false then
				SESS.SESS_DSLSignalResult = passstr
			else
				SESS.SESS_DSLSignalResult = failstr
			end
			SESS.SESS_DSLResult = failstr
		end
		if SESS.SESS_DSLResult == passstr then
			-- ATM checks
			if diag.IsAtmSupported() ~= false then
				SESS.SESS_ATMResult = failstr
				SESS.SESS_AtmSignalResult = failstr
				if diag.ATMCellIsWorking() ~= false and
					diag.ATMSyncIsWorking() ~= false then
					SESS.SESS_ATMResult = passstr
					SESS.SESS_AtmSignalResult = passstr
					if diag.PingATMSegment(1) ~= false then
						SESS.SESS_AtmOamSegPingResult = passstr
					else
						SESS.SESS_AtmOamSegPingResult = failstr
					end
					if diag.PingATMEndToEnd(1) ~= false then
						SESS.SESS_AtmOamEndPingResult = passstr
					else
						SESS.SESS_AtmOamEndPingResult = failstr
					end
				end
				if SESS.SESS_AtmSignalResult == passstr and (SESS.SESS_AtmOamSegPingResult == passstr or SESS.SESS_AtmOamEndPingResult == passstr) then
					SESS.SESS_ATMResult = passstr
				else
					SESS.SESS_ATMResult = failstr
				end
			end

			-- PTM checks
			if diag.IsPtmSupported() ~= false and functions.isTransportATM() ~= "yes" then
				if diag.IsPtmRunning() ~= false then
					SESS.SESS_PTMResult = passstr
				else
					SESS.SESS_PTMResult = failstr
				end
			end
			if diag.PingDefaultHost() ~= false then
				SESS.SESS_pingdfhost = passstr
			else
				SESS.SESS_pingdfhost = failstr
			end
		end --if DSL passed
	end

	-- wan checks
	if wan_conn ~= nil and wan_conn ~= "" then
	    local WANENET = functions.getWANPort()
		if (functions.isWANUP() and sdb.get("phy.dsl.state") == "DOWN") or
		    (sdb.get(string.format("system.sku-%s.cur-wan", sku)) == "ethernet") or
		    (WANENET ~= "" and sku == "72") then
			SESS.SESS_DSLResult = passstr --this is just to fake out the other tests
			SESS.SESS_AuthResult = skipped
		end
		if SESS.SESS_DSLResult == passstr then
			if diag.ConnIsPPP(wan_conn) ~= false then
				SESS.SESS_PPPoEResult = failstr
				SESS.SESS_AuthResult = failstr
				if diag.ConnPPPSubLinkIsUp(wan_conn) ~= false then
					SESS.SESS_PPPEtherSublinkResult = failstr
					if diag.PPPoEIsWorking(wan_conn) ~= false then
						SESS.SESS_PPPEtherSublinkResult = passstr
					end
					SESS.SESS_PPPConnectionResult = passstr
					if diag.PPPIsWaitingForAuth(wan_conn) ~= false then
						SESS.SESS_PPPConnectionResult = failstr
					end
					if SESS.SESS_PPPEtherSublinkResult == passstr and
						SESS.SESS_PPPConnectionResult == passstr then
						SESS.SESS_PPPoEResult = passstr
						SESS.SESS_AuthResult = passstr
					end
				end -- ppp sub link up
			else
				-- conn not ppp. static or dhcpc
				if diag.IsSystemSupplicantEnabled() ~= false then
					SESS.SESS_AuthResult = failstr
					if diag.IsConnLinkSupplicantAuthorized(wan_conn) ~= false then
						SESS.SESS_AuthResult = passstr
					end
				end
			end
			if SESS.SESS_AuthResult ~= failstr and
				SESS.SESS_PPPoEResult ~= failstr then
				-- Fix. don't check diag.EtherIsWorking(wan_conn)
				-- as that function needs work
				SESS.SESS_IPResult = failstr
				SESS.SESS_IPEthernetResult = failstr
				SESS.SESS_ValidWanIpResult = failstr
				SESS.SESS_IPPPPResult = failstr
				SESS.SESS_PingDfGatewayResult = failstr
				if diag.IPIsWorking(wan_conn) ~= false then
					SESS.SESS_IPPPPResult = passstr
				end
				if diag.BridgeIsWorking(wan_conn) ~= false and
					diag.IPIsWorking(lan_conn) ~= false and
					diag.IPIsWorking(wan_conn) ~= false then
					SESS.SESS_IPEthernetResult = passstr
					if diag.PingGatewayIPAddress() ~= false then
						SESS.SESS_PingDfGatewayResult = passstr
					end
					if diag.ConnIPIsValid(wan_conn) ~= false then
						SESS.SESS_ValidWanIpResult = passstr
					end
					if SESS.SESS_IPEthernetResult == passstr and
						SESS.SESS_PingDfGatewayResult == passstr and
						SESS.SESS_IPPPPResult == passstr and
						SESS.SESS_ValidWanIpResult == passstr then
						SESS.SESS_IPResult = passstr
					end
				end
			end
		end --DSL result == pass

		-- if we successfully reverse lookup our dns servers
		-- then dns is working.
		-- check the current dns servers, not the configured ones
		if SESS.SESS_IPResult == passstr then
			SESS.SESS_DNSResult = failstr
			local dns_server = sdb.get("ip.dns.curr-address1")
			if dns_server ~= nil and dns_server ~= "" then
				SESS.SESS_DNSRevLookupPriResult = failstr
				local l = sdb.run("ip.dns.lookup", dns_server)
				-- if we get back "Name or service not known", then
				-- we know the DNS server responded, and we have
				-- working DNS.
				if l[1] ~= nil and
				(
					string.find(l[1], "Invalid host name") == nil and
					string.find(l[1], "query timed out") == nil) then
					SESS.SESS_DNSRevLookupPriResult = passstr
				end
			else
				SESS.SESS_DNSRevLookupPriResult = "no primary dns server"
			end
			dns_server = sdb.get("ip.dns.curr-address2")
			if dns_server ~= nil and dns_server ~= "" then
				SESS.SESS_DNSRevLookupSecResult = failstr
				local l = sdb.run("ip.dns.lookup", dns_server)
				-- if we get back "Name or service not known", then
				-- we know the DNS server responded, and we have
				-- working DNS.
				if l[1] ~= nil and
				(
					string.find(l[1], "query timed out") == nil) then
					SESS.SESS_DNSRevLookupSecResult = passstr
				end
			else
				SESS.SESS_DNSRevLookupSecResult = "no secondary dns server"
			end
			if SESS.SESS_DNSRevLookupPriResult == passstr or
				SESS.SESS_DNSRevLookupSecResult == passstr then
				SESS.SESS_DNSResult = passstr
			end
		end

		-- 6rd test
		if diag.IPIsWorking(wan_conn) ~= false and
			ipv6ConnIsTypeRdCpe() == "1" then
			SESS.SESS_SixRdResult = failstr
			local oid = sdb.get("ip6.primary-wan-oid")
			local param = string.format("ip6.conn[%s].rd.continuity-test", oid)
			local l = sdb.run(param, "udp", "1280", "1")
			if l[1] ~= nil and
				string.find(l[1], "Sixrd Connection Active") ~= nil then
				SESS.SESS_SixRdResult = passstr
			end
		end
	end
end

function isRawMobile()
	if string.find(ENV.HTTP_USER_AGENT, "Mobile") ~= nil or
	string.find(ENV.HTTP_USER_AGENT, "iPod")~= nil or
	string.find(ENV.HTTP_USER_AGENT, "iPhone")~= nil or
	string.find(ENV.HTTP_USER_AGENT, "Android")~= nil or
	string.find(ENV.HTTP_USER_AGENT, "Symbian")~= nil then
		return true
	else
		return false
	end
end

function isMobile()
	if SESS.SESS_mobile_override == "1" then
		return false
	else
		return isRawMobile()
	end
end
function blockmyhost(myipaddr)
    local mymac = nil
    local RUNLIST = functions.get_lhd(true)
    local tbl = {}
    for i, val in ipairs (RUNLIST) do
        local iptable = {}
        iptable = functions.split(val, "|")
        if iptable[2] ~= nil  then
            if iptable[2] == myipaddr then
                mymac = iptable[4]
                break
            end
        end
    end
    if mymac ~= nil then
        local MYHOSTS = sdb.info("objlist", "restricted-hosts.host")
        for i, val in ipairs(MYHOSTS) do
            if mymac == sdb.get(string.format("%s.mac-addr", val)) then
                if sdb.get(string.format("%s.blocked", val)) ~= "no" then
                    SESS.SESS_access_time = sdb.get(string.format("%s.next-available", val))
                    SESS.SESS_blocked_type = sdb.get(string.format("%s.blocked", val))
                    return true
                end
            end
        end
    end
    return false
end
function find_best(stamax, macmatch, statematch, stationmatch)
                  for s = 1, stamax do
                      if stationmatch ~= s then
                          thismac = sdb.get(string.format("phy.wl80211[1].sta[%s].mac", s))
                          if thismac ~= "" then
                              thismac = string.lower(thismac)
                              if macmatch == thismac then
                                  thisstate = sdb.get(string.format("phy.wl80211[1].sta[%s].state", s))
                                  if statematch == "Disconnected" and thisstate == "Connected" then
                                      return s
                                  end
                              end
                          end
                      end
                  end
                return stationmatch
end

function how_many_bars(macaddr)
   local maxsta =  sdb.get("phy.wl80211.sta.objmax")
   if maxsta ~= nil then
       maxsta = tonumber(maxsta)
   else
       maxsta = 128
   end
   if sdb.get("phy.wl80211[1].state") == "operational" then
       for station = 1, maxsta do
          --5GHz macs in .sta are all uppercase, but 2.4GHz are all lowercase
          stamac = sdb.get(string.format("phy.wl80211[1].sta[%d].mac", station))
          if stamac ~= "" then stamac = string.lower(stamac) end
          if stamac == string.lower(macaddr) then
              --check if this is the best match
              usesta = station
              state = sdb.get(string.format("phy.wl80211[1].sta[%s].state", station))
              if state ~= "Connected" then
                  usesta = find_best(maxsta, stamac, state, station)
              end
              starssi = sdb.get(string.format("phy.wl80211[1].sta[%d].rssi", usesta))
              local mydb = tonumber(string.match(starssi, "[0-9,-]+"))
              if mydb == nil then
                  local norssi = string.format("Wi-Fi lookup of %s on station %s had no rssi", macaddr, usesta)
                  sdb.log(norssi)
                  return "1"
              elseif mydb <= -91 then return "1"
              elseif mydb <= -80 then return "2"
              elseif mydb <= -70 then return "3"
              elseif mydb <= -58 then return "4"
              else return "5"
              end
          end
       end
       myerr = string.format("Wi-Fi lookup of %s failed", macaddr)
       sdb.log(myerr)
       return "1"
   end
   myerr = "wireless is not currently operational, lookup of rssi failed"
   sdb.log(myerr)
   return "1"
end

function get_endnode(netresults)
    --numactual might be less than the number of phydiag entries
    --we expect. numlast is the sequence number of the last entry.
    --If a station doesn't respond, it's sequence number is skipped.
    --But the last sequence number could be missing, too.
    local node = 0
    curnum = sdb.get("phy.hpna.station-info.current-num-entries")
    if curnum == "" or string.find(curnum, "[^%d]") ~= nil then
        curnum = "1"
    end
    if netresults == "" then
        node = tonumber(curnum)
    else
        local numactual = sdb.get("phy.hpna.phy-diag.num-entries")
        if numactual == "" or string.find(numactual, "[^%d]") ~= nil then
            numactual = "1"
        end
        numactual = tonumber(numactual)
        local numresults = #LOG
        local numlast = LOG[numresults]
        numlast = tonumber(string.sub(numlast, 1, 3))
        if numlast ~= numactual then
            curnum = (curnum * (curnum-1)) - 1
            if numlast ~= curnum then numlast = curnum end
        end
        for i = 2, 10 do
            if i*(i-1) == numlast + 1 then
                node = i
                break
            end
        end
    end
   if node == 0 then node = 1 end
   return node
end

function port_warning(hostport)
    local mysku = sdb.get("system.sku")
    if sdb.get(string.format("system.sku-%s.cur-network", mysku)) == "ip-dsl" then return false end
    if mysku == "72" then return false end
    --warn in every case except ADSL
    local MATCHES = sdb.info("objlist", string.format("gfs[%s].rule[%s].match", SESS.GFSCHOSEN, SESS.RULE_OID))
    local isudp = false
    local istdp = false
    local intcheck = false
    local intmatchcheck = false --check it any matches set to ingress or egress
    local PORT = {}
    if MATCHES[1] ~= nil then
      for i, val in ipairs(MATCHES) do
        local matchval = sdb.get(string.format("%s.value", val))
        local matchcat = sdb.get(string.format("%s.category", val))
        if (matchcat == "in-link-oid" and matchval == "LAN") then
            intcheck = true
        elseif matchcat == "out-link-oid" and matchval == "WAN" then
            intcheck = true
        elseif matchcat == "in-link-oid" or matchcat == "out-link-oid" then
            intmatchcheck = true
        elseif matchval == "udp" or matchval == "17" then
            isudp = true
        elseif matchval == "tcp" or matchval == "6" then
            istcp = true
        end
      end
    end
    if intmatchcheck == true and intcheck == false then return false end
    local space = string.find(hostport, "%s")
    if space ~= nil then
        PORT[1] = string.sub(hostport, 1, space-1)
        PORT[2] = string.sub(hostport, space+1, 11)
        i = 1
        while true do
            local newport = tonumber(PORT[1]) + i
            if newport >= tonumber(PORT[2]) then break end
            PORT[i+2] = tostring(newport)
            i = i + 1
        end
    else
        PORT[1] = hostport
    end
    for i, val in ipairs(PORT) do
        if (val == "80" or val == "3389") and istcp then
            return true
        elseif (val == "47806" or val == "43962" or val == "69" or val == "123" or val == "53") and isudp then
            return true
        end
    end
    return false
end
function isProxySubnet()
    local MYLIST = sdb.info("template", "conn")
    for i, val in ipairs(MYLIST) do
        if string.find(val, "public-lan", 1, true) ~= nil then
            return true
        end
    end
    return false
end
function check10xwarn()
    if functions.isWANUP() then
        local wanconn = functions.get_path_from_oid_param("ip.primary-wan-oid")
        local wanip = sdb.get(string.format("%s.ipaddr", wanconn))
        if string.find(wanip, "10%.%d+%.%d+%.%d+") ~= nil then
            return true
        end
    end
    return false
end
function IsPSHEnabled()
	gfschosen = ""
	local GFSLIST = sdb.info("objlist_noorder", "gfs")
	for index,gfs in ipairs(GFSLIST) do
		local name = sdb.aget(gfs, "name")
		if name == "public-subnet-host" then
			gfschosen = gfs
		end
	end
	if gfschosen ~= "" then
	    RULES = sdb.info("objlist", string.format("%s.rules", gfschosen))
	    if sdb.get(string.format("%s.enable", gfschosen)) == "on"  then
	        return true
	    end
	end
	return false
end
function DisablePSH()
	local GFSLIST = sdb.info("objlist_noorder", "gfs")
	for index,gfs in ipairs(GFSLIST) do
		local name = sdb.aget(gfs, "name")
		if name == "public-subnet-host" then
			sdb.set(string.format("%s.enable", gfs), "off")
		end
	end
end

function getRestrictedIP()
    local restip = sdb.get("phy.wl80211[2].remote-ip")
    if restip == nil or restip == "" then
        restip = ""
    end
    return restip
end

function swallow(err, form)
    local page = ENV.REQUEST_URI
    if page == "/cgi-bin/dosprotect.ha" then
        err = "Flood limit must be 1-100"
    elseif page == "/cgi-bin/clusterconfig.ha" then
        local mtumin = "128"
        local IP6WANConn = functions.get_path_from_oid_param("ip6.primary-wan-oid")
        if functions.isIPv6Enabled() and sdb.get(string.format("%s.enable", IP6WANConn)) == "on" then
            mtumin = "1300"
        end
        err = "MTU depends upon IPv6 Configuration. MTU must be " ..  mtumin .. "-1500."
    elseif page == "/cgi-bin/syslog.ha" then
        if string.find(err, "Invalid value") ~= nil or
           (string.find(err, "A required setting is empty", 1, true) ~= nil and form == "location") then
            err = "IP Address must be valid IPv4 or IPv6 format"
        end
    elseif page == "/cgi-bin/ippass.ha" then
        if string.find(err, "Not an IP address", 1, true) ~= nil or
           (string.find(err, "A required setting is empty", 1, true) ~= nil and form == "defsrvint") then
            err = "IP Address must be nnn.nnn.nnn.nnn"
        elseif string.find(err, "Number is too large", 1, true) ~= nil or
               string.find(err, "Number is too small", 1, true) ~= nil then
            err = "DHCP lease time must be between 3 minutes and 99 days"
        end
    elseif page == "/cgi-bin/wconfig.ha" then
        if form == "power" and
        ((string.find(err, "Number is too", 1, true) ~= nil) or
        (string.find(err, "Invalid range", 1, true) ~= nil) or
        (string.find(err, "Not a number", 1, true) ~= nil))  then
            err = "Power Level must be 1-100"
        elseif (form == "ssidname") then
            if (err == "String is too short") then
                err = "Network Name must be at least one character"
            else
                err = "Network Name cannot contain some special characters"
            end
        elseif string.find(err, "Invalid key length", 1, true) ~= nil or
               string.find(err, "A required setting is empty") ~= nil then
            if (form == "key1" and FORM.security == "wep") or
               (form == "key2" and FORM.security2 == "wep") then
                err = "Password must contain the number of characters specified in WEP Password Length"
            elseif (form == "key1" and FORM.security == "wpa") or
                (form == "key2" and FORM.security2 == "wpa") then
                err = "Password must contain 8-63 characters"
            end
        elseif string.find(err, "Invalid characters in string", 1, true) ~= nil and
                (form == "key1" or form == "key2") then
            err = "WEP Password characters must be 0-9, A-F"
        end
    elseif page == "/cgi-bin/dhcpserver.ha" then
        if string.find(err, "Number is too large", 1, true) ~= nil or
               string.find(err, "Number is too small", 1, true) ~= nil then
            err = "DHCP lease time must be between 3 minutes and 99 days"
        elseif string.find(err, "Not an IP address", 1, true) ~= nil or
               string.find(err, "is an invalid IP address", 1, true) ~= nil then
            err = "IP Address must be nnn.nnn.nnn.nnn"
        elseif string.find(err, "nexthop address cannot be", 1, true) ~= nil then
            err = "address must be in LAN subnet range"
        elseif string.find(err, "A required setting is empty") ~= nil then
            if form == "ipaddr" or form == "pubipaddr" or form == "cripaddr" or form == "delgate" then
                err = "IP Address must be nnn.nnn.nnn.nnn"
            end
        end
    elseif page == "/cgi-bin/packetfilteredit.ha" then
        if (string.find(err, "is an invalid IP address or set of IP addresses", 1, true) ~= nil) or
           ((string.find(err, "A required setting is empty", 1, true) ~= nil) and
           (form == "ipaddr")) then
            err = "The format of IP Address should be nnn.nnn.nnn.nnn or nnn.nnn.nnn.nnn-nnn.nnn.nnn.yyy or nnn.nnn.nnn.0/y"
        elseif SESS.category == "src-host-mac" or SESS.category == "dst-host-mac" then
            err = "MAC addresses must be xx:xx:xx:xx:xx:xx"
        elseif SESS.category ==  "icmp-type" then
            err = "ICMP Type must be 0-255"
        elseif SESS.category == "ip-proto" and SESS.value == "by name" then
            err = "Click hyperlink list for valid protocol names"
        end
    end
    return err
end
function setonly(parm, formname, formvalue, sesserr, valc)
    local err
    param_match[formname] = parm
    if ((formvalue == "") and
        (formname == "ipaddr" or
         formname == "pubipaddr" or
         formname == "crmask" or
         formname == "pubmask" or
         formname == "ipmask" or
         formname == "cripaddr" or
         formname == "delgate" or
         formname == "pubdhcpstart" or
         formname == "pubdhcpend" or
         formname == "dhcpstart" or
         formname == "dhcpend")) then
            err = "IP Address must be nnn.nnn.nnn.nnn"
    elseif ((formvalue == "0.0.0.0") and
         (formname == "ipaddr" or
         formname == "pubipaddr" or
         formname == "crmask" or
         formname == "pubmask" or
         formname == "ipmask" or
         formname == "cripaddr" or
         formname == "pubdhcpstart" or
         formname == "pubdhcpend" or
         formname == "dhcpstart" or
         formname == "dhcpend")) then
            err = "IP Address must be valid in the form nnn.nnn.nnn.nnn"
    else
        err = sdb.set(parm, formvalue)
    end
    if err ~= "" then
        err = swallow(err, formname)
        sesserr = sesserr .. formname .. "{%" .. err .. "%}"
    end
    return err, sesserr, valc
end

function setparm(parm, formname, formvalue, sesserr, valc)
    local err = sdb.set(parm, formvalue)
    if err == "" then
        res = sdb.validate()
        if (res ~= 0) then --validation failed
            for i, valerr in ipairs(ERROR) do
                if valc == i then
                    err = valerr
                    valc = valc + 1
                end
                ERROR[i] = nil
            end
        end
    end
    if err ~= "" then
        err = swallow(err, formname)
        sesserr = sesserr .. formname .. "{%" .. err .. "%}"
    end
    return err, sesserr, valc
end

function pullerr(formname, sesserr, class)
    -- format is formname{%errmsg%}formname{%errmsg%}
    local myclass = ""
    if class ~= nil then
       myclass = string.format(" class=\"%s\"", class)
    end
    local tmp = formname .. "{%"
    local errtab = {}
    errtab = functions.split(sesserr, "}")
    for i, err in ipairs(errtab) do
        local start, finish = string.find(err, tmp, 1, true)
        if start == 1 then
            return "<em" .. myclass ..">" .. string.sub(err, finish+1, string.find(err, "%", finish+1, true)-1) .. "</em>"
        end
    end
    return ""
end

function wifiap(lhdssid, format)
    --lhdssid will be ssid-1, ssid-2, ssid2-1, ssid2-2, or ssid2-3
    --capture the first digit to find which radio
    radionum = string.match(lhdssid, "%a+(%d)%-%d")
    --capture the second digit to determine which ssid
    if radionum == nil then radionum = "1" end
    ssidnum = string.match(lhdssid, "%w%-(%d)")
    if ssidnum == nil then ssidnum = "1" end
    local ssidtype = "Home"
    if ssidnum == "2" then
        ssidtype = "Guest"
    elseif ssidnum == "3" then
        ssidtype = "Video"
    end
    local radioname = sdb.get(string.format("phy.wl80211[%s].band-lock", radionum))
    local ssidname = sdb.get(string.format("phy.wl80211[%s].ssid[%s].ssid-name", radionum, ssidnum))
    if format == "1" then
        accesspt = string.format("%s GHz, %s, %s", radioname, ssidtype, ssidname)
    else
        accesspt = string.format("<br />%s GHz<br />Type: %s<br />Name: %s", radioname, ssidtype, ssidname)
    end
    return accesspt
end
function isFiberSupported()
    local model = sdb.get("system.model-name")
    if model == "NVG595" or  model == "NVG3200" then
        return true
    else
        return false
    end
end
